@@ -32,23 +32,16 @@ Mainly the RestClient, which you can use to easily pythonify a rest api.
 based on https://github.com/jpaugh/agithub/commit/1e2575825b165c1cb7cbd85c22e2561fc4d434d3
 
 @author: Jonathan Paugh
-@author: Jens Timmerman
+@author: Jens Timmerman (Ghent University)
+@author: Kenneth Hoste (Ghent University)
 """
 import base64
 import copy
-import urllib
-import urllib2
-try:
-    import json
-except ImportError:
-    import simplejson as json
+import json
+from functools import partial
 
 from vsc.utils import fancylogger
-
-try:
-    from functools import partial
-except ImportError:
-    from vsc.utils.missing import partial
+from vsc.utils.py2vs3 import HTTPSHandler, Request, build_opener, urlencode
 
 
 class Client(object):
@@ -92,8 +85,8 @@ class Client(object):
         else:
             self.user_agent = user_agent
 
-        handler = urllib2.HTTPSHandler()
-        self.opener = urllib2.build_opener(handler)
+        handler = HTTPSHandler()
+        self.opener = build_opener(handler)
 
         if username is not None:
             if password is None and token is None:
@@ -196,7 +189,7 @@ class Client(object):
     def urlencode(self, params):
         if not params:
             return ''
-        return '?' + urllib.urlencode(params)
+        return '?' + urlencode(params)
 
     def hash_pass(self, password, username=None):
         if not username:
@@ -208,7 +201,7 @@ class Client(object):
             sep = '/'
         else:
             sep = ''
-        request = urllib2.Request(self.url + sep + url, data=body)
+        request = Request(self.url + sep + url, data=body)
         for header, value in headers.iteritems():
             request.add_header(header, value)
         request.get_method = lambda: method
